#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "s2plot.h"


void wl2RGB(float wl, double *r, double *g, double *b)
/*
c      NetPBM Software: ftp://ftp.cs.ubc.ca/ftp/archive/netpbm/
*/
{
   float R = 1.0, G = 1.0, B = 1.0;
   float gamma=.80;
   float SSS;

   if ((wl > 380) && (wl < 440)) {
      R = -1.*(wl-440.)/(440.-380.);
      G = 0.;
      B = 1.;
   } else if ((wl >= 440) && (wl <= 490)) {
      R = 0.;
      G = (wl-440.)/(490.-440.);
      B = 1.;
   } else if ((wl >= 490) && (wl <= 510)) {
      R = 0.;
      G = 1.;
      B = -1.*(wl-510.)/(510.-490.);
   } else if ((wl >= 510) && (wl <= 580)) {
      R = (wl-510.)/(580.-510.);
      G = 1.;
      B = 0.;
   } else if ((wl >= 580) && (wl <= 645)) {
      R = 1.;
      G = -1.*(wl-645.)/(645.-580.);
      B = 0.;
   } else if ((wl >= 645) && (wl <= 780)) {
      R = 1.;
      G = 0.;
      B = 0.;
   }

/* Fix the end-points */

   if (wl > 700) {
      SSS=.3+.7* (780.-wl)/(780.-700.);
      SSS = 1.0;
   } else if (wl < 420) {
      SSS=.3+.7*(wl-380.)/(420.-380.);
      SSS = 1.0;
   } else {
      SSS=1.;
   }

   *r = pow((SSS*R), gamma);
   *g = pow((SSS*G), gamma);
   *b = pow((SSS*B), gamma);

}



int main(int argc, char *argv[])
{
   FILE *fp;			/* File to read from */
   char string[128];		/* Line of text from file */
   float wl, y1, y2, z = 0;	/* Input data */
   COLOUR col;			/* Colour to plot point point with */
   int pt = 1;			/* Projection type: orthographic = 1 */

   s2opend("/?",argc,argv);	/* Open the S2PLOT display */
   s2svp(-2,2,-1,1,-0.1,0.1);	/* Set the screen volume */
   s2swin(250,950,0.0,100.0,-1.0,1.0);	/* Set the window coordinates */
   ss2spt(pt);			/* Set projection type */
  
   s2sci(S2_PG_BLUE);			/* Set label plotting colour */
   s2box("BTMNOPQ",0,0,"BTMNOPQ",0,0,"",0,0);		/* Draw box */
   s2lab("Wavelength (nm)","Transmission (%)","","");	/* Label it */

   fp = fopen("filters.dat","r");	/* Open the file */
   fgets(string, 128, fp);		/* Read a line of text */

   while (!feof(fp)) {		
      if (sscanf(string,"%f %f %f",&wl, &y1, &y2) == 3) {
					/* Two filters on this line */
	    wl /= 10.0;			/* Convert wavelength to nm */
            wl2RGB(wl, &col.r, &col.g, &col.b);	/* Get RGB colour */

            ns2point(wl, y1, z, col.r, col.g, col.b);		
            ns2point(wl, y2, z, col.r, col.g, col.b);

      } else if (sscanf(string,"%f %f",&wl, &y1) == 2) {
					/* One filter on this line */
	 wl /= 10.0;			/* Convert wavelength to nm */
         wl2RGB(wl, &col.r, &col.g, &col.b);
         ns2point(wl, y1, z, col.r, col.g, col.b);
      }
      fgets(string, 128, fp);		/* Get next line of text */
   }
   fclose(fp);				/* Close the file */

/**** Draw some labels on the plot ****/

   wl2RGB(350.0, &col.r, &col.g, &col.b);
   s2scr(32, col.r, col.g, col.b);
   s2sci(32);
   s2textxy(330.0, 85.0, 0.0, "U");




/* Set the camera position so that we get a nicely centred graph */
   XYZ pos = { 0.0, 0.0, 5.5 };
   XYZ vd = { 0, 0, -1 };
   XYZ vu = { 0, 1, 0 };
   ss2sc(pos, vu, vd, 0);

   s2show(1);
   return 1;
}

