#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "s2plot.h"

enum Magnitudes { Vmag, BV, UB, VR, RI, VI };
char labels[6][5] = { "Vmag","B-V", "U-B", "V-R", "R-I", "V-I" };

typedef struct {
   char name[16];
   float rah, ram, ras;
   float ded, dem, des;
   float mags[6];
} Data;

typedef struct {
   float min, max;
} MinMax;

typedef struct {
   MinMax xmm, ymm, zmm;
   int xaxis, yaxis, zaxis;
   int pt;
} PlotData;

Data readData(char *string) 
{
   int namelen = 12;
   Data data;
   char *ptr = string+namelen;
   int i;

   for (i=0;i<namelen;i++) {
      data.name[i] = string[i];
   }
   data.name[i] = '\0';

   sscanf(ptr, "%f %f %f %f %f %f %f %f %f %f %f %f",
        &data.rah, &data.ram, &data.ras, 
	&data.ded, &data.dem, &data.des,
	&data.mags[Vmag], 
	&data.mags[BV], &data.mags[UB], &data.mags[VR], 
	&data.mags[RI], &data.mags[VI]);	

   return data;
}

MinMax dataMinMax(Data *data, int N, int idx)
{
   int i;
   MinMax mm;

   mm.min = data[0].mags[idx];	/* First data value */
   mm.max = data[0].mags[idx];	/* First data value */
   for (i=1;i<N;i++) {
      if (data[i].mags[idx] < mm.min) mm.min = data[i].mags[idx];
      if (data[i].mags[idx] > mm.max) mm.max = data[i].mags[idx];
   }
   return mm;
}

int countLines(FILE *fp) 
{
   int N = 0;				/* Initialise counter */
   char string[128];			/* Text line from file */

   fgets(string, 128, fp);		/* Read a line */
   while (!feof(fp)) {			/* Not end of file */
      N++; 				/* Count this line */
      fgets(string, 128, fp);		/* Read next line */
   }
   rewind(fp);				/* Reset file to the start */
   return N;				/* Return number of lines */ 
}


PlotData initialisePlotData(int xaxis, int yaxis, int zaxis, int pt, Data *data, int N)
{
   PlotData pd;
   pd.xaxis = xaxis;
   pd.yaxis = yaxis;
   pd.zaxis = zaxis;
   pd.pt    = pt;			/* Orthographic = 1 */
					/* Perspective  = 0 */

   pd.xmm = dataMinMax(data, N, xaxis);
   pd.ymm = dataMinMax(data, N, yaxis);
   pd.zmm = dataMinMax(data, N, zaxis);

   return pd;
}

int main(int argc, char *argv[])
{
   FILE *fp;
   char string[128];
   int i, N = -1; 
   Data *data;
   PlotData pd;
   XYZ xyz;

   fp = fopen("table2.dat","r"); 
   N  = countLines(fp);

   data = (Data *)calloc(N, sizeof(Data));	/* Allocate memory */

   for (i=0;i<N;i++) {
      fgets(string, 128, fp);
      data[i] = readData(string);
   }
   fclose(fp);

   pd = initialisePlotData(Vmag, BV, UB, 0, data, N);

   s2opend("/?",argc,argv);
   ss2spt(pd.pt);
   s2swin(pd.xmm.min,pd.xmm.max, pd.ymm.min,pd.ymm.max, pd.zmm.min,pd.zmm.max);
   s2sci(S2_PG_BLUE);
   s2box("BCDETMNOPQ",0,0,"BCDETMNOPQ",0,0,"BCDETMNOPQ",0,0);
   s2lab(labels[pd.xaxis],labels[pd.yaxis],labels[pd.zaxis],"");


   s2sci(S2_PG_WHITE);
   for (i=0;i<N;i++) {
      xyz.x = data[i].mags[pd.xaxis];
      xyz.y = data[i].mags[pd.yaxis];
      xyz.z = data[i].mags[pd.zaxis];

      s2pt1(xyz.x, xyz.y, xyz.z, 1);
   }
   
   s2show(1);
   return 1;
}

