#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "s2plot.h"

enum Magnitudes { Vmag, BV, UB, VR, RI, VI };
char labels[6][5] = { "Vmag","B-V", "U-B", "V-R", "R-I", "V-I" };

typedef struct {
   char name[16];
   float rah, ram, ras;
   float ded, dem, des;
   float mags[6];
} Data;

typedef struct {
   float min, max;
} MinMax;

typedef struct {
   MinMax xmm, ymm, zmm, cmm;
   int xaxis, yaxis, zaxis, caxis;
   int pt;
} PlotData;

int *sel;

void hcb(int *id)
{
   cs2th(*id);					/* Toggle state of handle */
   sel[*id] = (sel[*id] + 1) % 2;		/* Toggles 0/1 in selection array */
}


Data *global_data;
int  global_N = -1;
PlotData global_pd;

void cb(double *t, int *kc)
{
   int i, ci;
   float scale;
   COLOUR col = { 1.0, 1.0, 1.0 };
   COLOUR hil = { 1.0, 1.0, 0.0 };
   XYZ xyz;
   float hsize = 0.02;

   if (global_N < 0) {
      fprintf(stderr,"Global value has not been set\n");
      exit(0);
   }

   scale = 15.0/(global_pd.cmm.max - global_pd.cmm.min);
   s2sch(1.5);
   for (i=0;i<global_N;i++) {
      xyz.x = global_data[i].mags[global_pd.xaxis];
      xyz.y = global_data[i].mags[global_pd.yaxis];
      xyz.z = global_data[i].mags[global_pd.zaxis];

      ci = (int)((global_data[i].mags[global_pd.caxis]-global_pd.cmm.min)*scale + 1);
      s2sci(ci);
      s2pt1(xyz.x, xyz.y, xyz.z, 1);

      cs2ah(xyz, hsize, col, hil, i, sel[i]);

      s2sci(S2_PG_WHITE);
      if (sel[i]) {
         if (cs2qhv()) 
            s2textxy(xyz.x, xyz.y, xyz.z, global_data[i].name);   
      }
   }

}



Data readData(char *string) 
{
   int namelen = 12;
   Data data;
   char *ptr = string+namelen;
   int i;

   for (i=0;i<namelen;i++) {
      data.name[i] = string[i];
   }
   data.name[i] = '\0';

   sscanf(ptr, "%f %f %f %f %f %f %f %f %f %f %f %f",
        &data.rah, &data.ram, &data.ras, 
	&data.ded, &data.dem, &data.des,
	&data.mags[Vmag], 
	&data.mags[BV], &data.mags[UB], &data.mags[VR], 
	&data.mags[RI], &data.mags[VI]);	

   return data;
}

MinMax dataMinMax(Data *data, int N, int idx)
{
   int i;
   MinMax mm;

   mm.min = data[0].mags[idx];	/* First data value */
   mm.max = data[0].mags[idx];	/* First data value */
   for (i=1;i<N;i++) {
      if (data[i].mags[idx] < mm.min) mm.min = data[i].mags[idx];
      if (data[i].mags[idx] > mm.max) mm.max = data[i].mags[idx];
   }
   return mm;
}

int countLines(FILE *fp) 
{
   int N = 0;				/* Initialise counter */
   char string[128];			/* Text line from file */

   fgets(string, 128, fp);		/* Read a line */
   while (!feof(fp)) {			/* Not end of file */
      N++; 				/* Count this line */
      fgets(string, 128, fp);		/* Read next line */
   }
   rewind(fp);				/* Reset file to the start */
   return N;				/* Return number of lines */ 
}


PlotData initialisePlotData(int xaxis, int yaxis, int zaxis, int caxis,
		int pt, Data *data, int N)
{
   PlotData pd;
   pd.xaxis = xaxis;
   pd.yaxis = yaxis;
   pd.zaxis = zaxis;
   pd.caxis = caxis;
   pd.pt    = pt;			/* Orthographic = 1 */
					/* Perspective  = 0 */

   pd.xmm = dataMinMax(data, N, xaxis);
   pd.ymm = dataMinMax(data, N, yaxis);
   pd.zmm = dataMinMax(data, N, zaxis);
   pd.cmm = dataMinMax(data, N, caxis);

   return pd;
}

int main(int argc, char *argv[])
{
   FILE *fp;
   char string[128];
   int i, N = -1;
   Data *data;
   PlotData pd;

   fp = fopen("table2.dat","r"); 
   N  = countLines(fp);

   data = (Data *)calloc(N, sizeof(Data));	/* Allocate memory */

   for (i=0;i<N;i++) {
      fgets(string, 128, fp);
      data[i] = readData(string);
   }
   fclose(fp);

   pd = initialisePlotData(Vmag, BV, UB, VR, 0, data, N);


   s2opend("/?",argc,argv);
   ss2spt(pd.pt);
   s2swin(pd.xmm.min,pd.xmm.max, pd.ymm.min,pd.ymm.max, pd.zmm.min,pd.zmm.max);
   s2sci(S2_PG_BLUE);
   s2box("BCDETMNOPQ",0,0,"BCDETMNOPQ",0,0,"BCDETMNOPQ",0,0);
   s2lab(labels[pd.xaxis],labels[pd.yaxis],labels[pd.zaxis],
	"Shift-S to toggle handles, Shift-c to toggle cross-hairs");

   global_data = data;
   global_N = N;  
   global_pd = pd;
  
   cs2scb(cb);
   
   sel = (int *)calloc(N, sizeof(int));
   for (i=0;i<N;i++) sel[i] = 0;
   cs2shcb(hcb);
   

   s2show(1);
   return 1;
}

