#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "s2plot.h"


/* CONSTANTS */
#define D2R  0.017453292519943295       /* Conversion of degrees to radians */
#define RA2R 0.261799387799149408       /* Conversion of RA (hrs) to radians */
#define R2D 57.295779513082322865       /* Conversion of radians to degrees */

#define DEMIN                   -90.0           /* Maximum Declination in degrees */
#define DEMAX                   +90.0           /* Maximum Declination in degrees */
#define RAMIN                     0.0           /* Minimum RA value in hours */
#define RAMAX                    24.0           /* Maximum RA value in hours */

#define FLAT  0
#define SPACE 1 

enum Magnitudes { Vmag, BV, UB, VR, RI, VI };
char labels[6][5] = { "Vmag","B-V", "U-B", "V-R", "R-I", "V-I" };

typedef struct {
   char name[16];
   float rah, ram, ras, ra;
   float ded, dem, des, de;
   XYZ xyz;
   float mags[6];
} Data;

typedef struct {
   float min, max;
} MinMax;

void cb(double *t, int *kc)
{
   XYZ pos = { 0.0, 0.0, 4.5 };
   XYZ vu  = { 0.0, 1.0, 0.0 };
   XYZ vd  = { 0.0, 0.0, -1.0 };
   ss2sc(pos, vu, vd, 0);
}

Data readData(char *string) 
{
   int namelen = 12;
   Data data;
   char *ptr = string+namelen;
   int i;

   for (i=0;i<namelen;i++) {
      data.name[i] = string[i];
   }
   data.name[i] = '\0';

   sscanf(ptr, "%f %f %f %f %f %f %f %f %f %f %f %f",
        &data.rah, &data.ram, &data.ras, 
	&data.ded, &data.dem, &data.des,
	&data.mags[Vmag], 
	&data.mags[BV], &data.mags[UB], &data.mags[VR], 
	&data.mags[RI], &data.mags[VI]);	

   return data;
}

MinMax dataMinMax(Data *data, int N, int idx)
{
   int i;
   MinMax mm;

   mm.min = data[0].mags[idx];	/* First data value */
   mm.max = data[0].mags[idx];	/* First data value */
   for (i=1;i<N;i++) {
      if (data[i].mags[idx] < mm.min) mm.min = data[i].mags[idx];
      if (data[i].mags[idx] > mm.max) mm.max = data[i].mags[idx];
   }
   return mm;
}

int countLines(FILE *fp) 
{
   int N = 0;				/* Initialise counter */
   char string[128];			/* Text line from file */

   fgets(string, 128, fp);		/* Read a line */
   while (!feof(fp)) {			/* Not end of file */
      N++; 				/* Count this line */
      fgets(string, 128, fp);		/* Read next line */
   }
   rewind(fp);				/* Reset file to the start */
   return N;				/* Return number of lines */ 
}


XYZ rade2xyz(float ra, float dec, float radius)
/* Converts a RA (0..24) and Dec (-90..+90) to an XYZ point */
{
   XYZ xyz;
   double p, t, rsinp;
       
   p = (DEMAX - dec)*D2R;               /* Convert to radians */
   t = ra*RA2R;                         /* Convert to radians */

   rsinp =  radius * sin(p);            /* Save one calculation */
   xyz.x =  rsinp  * cos(t);
   xyz.z = -rsinp  * sin(t);
   xyz.y =  radius * cos(p);

   return xyz;
}


int main(int argc, char *argv[])
{
   FILE *fp;
   char string[128];
   int i, N = -1;
   Data *data;
   int pt = 1;				/* Orthographic */
   int mode = SPACE;
   
   fp = fopen("table2.dat","r"); 
   N  = countLines(fp);

   data = (Data *)calloc(N, sizeof(Data));	/* Allocate memory */

   for (i=0;i<N;i++) {
      fgets(string, 128, fp);
      data[i] = readData(string);
      data[i].ra = data[i].rah + (data[i].ram + data[i].ras/60.0)/60.0;
      data[i].de = data[i].ded + (data[i].dem + data[i].des/60.0)/60.0;
      data[i].xyz = rade2xyz(data[i].ra, data[i].de, 1.0);
   }
   fclose(fp);

   s2opend("/?",argc,argv);

   if (mode == FLAT) {
      s2svp(-2,2,-1,1,-1,1);
      s2swin(0.0, 24.0, -90.0, +90.0, -0.1, 0.1);
      s2sch(2);
      s2sci(S2_PG_BLUE);
      s2box("BCDENMOPTQ",2,0,"BCDENMOPQT",30,0,"BCDENMOPTQ",0,0);
      s2lab("Right Ascension (hours)","Declination (degrees)","","");
      ss2spt(pt);

      s2sci(S2_PG_WHITE);
      for (i=0;i<N;i++) { 
         s2pt1(data[i].ra, data[i].de, 0.0, 1);
      }
   
      s2scb(cb);
      s2sci(S2_PG_LTGREY);
      ns2line(0.0, 0.0, 0.0, 24.0, 0.0, 0.0, 0.2, 0.2, 0.2);

   } else if (mode == SPACE) {
      s2swin(-1,1, -1,1, -1,1);
      for (i=0;i<N;i++) { 
         s2pt1(data[i].xyz.x, data[i].xyz.y, data[i].xyz.z, 1);
      }
   }

   s2show(1);
   return 1;
}

