#include <stdio.h>
#include <stdlib.h>
#include "s2plot.h"

int main(int argc, char *argv[])
{
   float xmin = -1, xmax = +1;          /* X data range */
   float ymin = -1, ymax = +1;          /* Y data range */
   float zmin = -1, zmax = +1;          /* Z data range */
   float x, y, z;                       /* Data coordinate */
   float nx, ny, nz;                    /* Normal vector */
   float radius;                        /* Radius of sphere */
   float rin, rout;                     /* Inner/outer radii of annulus */
   float red, green, blue;              /* Colours */
   int nseg;                            /* Number of segments in circle */
   float asp;                           /* Aspect ratio of circle */
   int flipx, flipy;                    /* How should text be flipped? */

   s2opend("/?",argc,argv);             /* Which S2PLOT device to open? */
   s2swin(xmin, xmax, ymin, ymax, zmin, zmax);  /* Set the display coordinates */
   s2box("BCDE",0,0,"BCDE",0,0,"BCDE",0,0);     /* Draw a coordinate box */

   x = 0.0;                             /* Data location */
   y = 0.0;
   z = 0.0;

/*** Drawing rectangles ***/
   radius = 0.5;                        /* Rectangle size */
   rout = 0.9;
   s2sci(S2_PG_BLUE);                   /* A blue rectangle */
   s2rectxz(x-rout,x+rout, z-rout,z+rout, y-radius);

/*** Drawing spheres ***/
   ss2ssr(32);                          /* Set the resolution of a sphere */
   red   = 0.9;                         /* RGB colour values between [0..1] */
   green = 0.2;
   blue  = 0.5;
   ns2sphere(x, y, z, radius, red, green, blue);

/*** Drawing circles and simple annuli ***/
   y = 0.7;                             /* Change the y-coordinte */
   nseg = 100;                          /* Draw a very smooth circle */
   asp = 1.0;                           /* Draw a circle, not squashed ellipse */
   rout   = 0.2;
   rin    = 0.1;
   radius = 0.3;                        /* Radius of circle */
   s2sci(S2_PG_GREEN);
   s2circxy(x, y, z, radius, nseg, asp);        /* Circle in x-y plane */
   s2diskxy(x, y, z, rin, rout);                /* Annulus in x-y plane */
   s2sci(S2_PG_YELLOW);
   s2circxz(x, y, z, radius, nseg, asp);        /* Circle in x-z plane */
   s2diskxz(x, y, z, rin, rout);                /* Annulus in x-z plane */
   s2sci(S2_PG_CYAN);
   s2circyz(x, y, z, radius, nseg, asp);        /* Circle in y-z plane */
   s2diskyz(x, y, z, rin, rout);                /* Annulus in y-z plane */

/*** Drawing more general annuli ***/
   y     = 0.0;                         /* Change the y-coordinte */
   rin   = 0.7;                         /* Inner radius */
   rout  = 0.9;                         /* Outer radius */
   red   = 0.1;                         /* RGB colour values between [0..1] */
   green = 0.7;
   blue  = 0.4;
   nx = 0.3;                            /* Coordinates of normal vector */
   ny = 0.8;
   nz = 0.2;
   ns2disk(x, y, z, nx, ny, nz, rin, rout, red, green, blue);

/*** Drawing normal and flipped text ***/
   x = 0.0;                             /* New coordinates for text */
   y = -0.8;
   z = 0.0;
   s2textxy(x, y, z, "S2PLOT shapes");  /* Text in x-y plane */

   flipx = +1;                                  /* Text along x-direction */
   flipy = -1;                                  /* Flip on y-axis */
   y = -0.90;                                   /* A small offset on y-axis */
   s2textxyf(x, y, z, flipx, flipy, "S2PLOT shapes");   /* Flipped text! */

/*** Drawing arrows ***/
   x = -1.0;                            /* Set new coordinates */
   y = -0.85;
   z =  0.0;
   s2sci(S2_PG_ORANGE);                 /* Set the colour to orange */
   s2sch(2);                            /* Set the character height */
                                        /*   - sets arrowhead size */
   s2arro(x, y, z, 0, y, z);            /* Draw an arrow */

   s2show(1);                           /* Hand over to the display */

   return 1;
}
